/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_AREALENSINVERSIONGAFACTORYBASE_H

#define GRALE_AREALENSINVERSIONGAFACTORYBASE_H

#include "graleconfig.h"
#include "randomnumbergenerator.h"
#include <mogal/gafactory.h>

namespace grale
{

class AreaLensInversionGAFactoryParams;

// NOTE: virtual inheritance is necessary!
class GRALE_IMPORTEXPORT AreaLensInversionGAFactoryBase : public virtual mogal::GAFactory
{
public:
	AreaLensInversionGAFactoryBase();
	~AreaLensInversionGAFactoryBase();

	// TODO: should be implemented again in derived class if necessary
	mogal::GAFactoryParams *createParamsInstance() const;
	
	const mogal::GAFactoryParams *getCurrentParameters() const;

	bool init(const mogal::GAFactoryParams *p);

	const RandomNumberGenerator *getRandomNumberGenerator() const				{ return &m_rndGen; }
protected:
	virtual bool subInit(const AreaLensInversionGAFactoryParams *p)				{ return true; }
	int getMaximumNumberOfGenerations() const						{ return maxgenerations; }
private:
	RandomNumberGenerator m_rndGen;
	AreaLensInversionGAFactoryParams *currentparams;
	int maxgenerations;
};
	
} // end namespace

#endif // GRALE_AREALENSINVERSIONGAFACTORYBASE_H

