/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_GRIDLENSINVERSIONGAFACTORYPARAMSNEW_H

#define GRALE_GRIDLENSINVERSIONGAFACTORYPARAMSNEW_H

#include "graleconfig.h"
#include "lensinversiongafactoryparams.h"
#include "grid.h"
#include <list>

namespace grale
{

class ImagesDataExtended;
class GravitationalLens;

class GRALE_IMPORTEXPORT GridLensInversionGAFactoryParamsNew : public LensInversionGAFactoryParams
{
public:
	enum BasisFunctionType { PlummerBasis, SquareBasis, GaussBasis };
	enum MassSheetSearchType { None, Genome };
	
	GridLensInversionGAFactoryParamsNew();
	GridLensInversionGAFactoryParamsNew(int maxGenerations,
					 const std::list<ImagesDataExtended *> &images, 
			                 const std::list<GridSquare> &gridSquares, 
					 double lensDistance,
					 double lensRedshift,
					 double initialMassScale,
					 bool copyImages,
					 bool allowNegativeValues,
					 const GravitationalLens *pBaseLens,
					 BasisFunctionType basisFunction,
					 MassSheetSearchType sheetSearchType,
					 bool showEvolution = false,
					 bool saveFirstGenome = false);
	~GridLensInversionGAFactoryParamsNew();

	int getMaximumNumberOfGenerations() const								{ return m_maxGenerations; }
	double getLensDistance() const										{ return m_lensDistance; }
	double getLensRedshift() const										{ return m_lensRedshift; }
	double getInitialMassScale() const									{ return m_initialMassScale; }
	const std::list<ImagesDataExtended *> &getImages() const						{ return m_images; }
	const std::list<GridSquare> &getGridSquares() const							{ return m_gridSquares; }
	bool allowNegativeValues() const									{ return m_allowNegative; }
	const GravitationalLens *getBaseLens() const								{ return m_pBaseLens; }
	BasisFunctionType getBasisFunctionType() const								{ return m_basisFunctionType; }
	MassSheetSearchType getMassSheetSearchType() const							{ return m_massSheetSearchType; }
	bool getShowEvolution() const										{ return m_showEvolution; }
	bool getSaveFirstGenome() const										{ return m_saveFirstGenome; }

	GridLensInversionGAFactoryParamsNew *createCopy(bool copyImages) const;
	
	bool write(serut::SerializationInterface &si) const;
	bool read(serut::SerializationInterface &si);
private:
	void copyFieldsFrom(const GridLensInversionGAFactoryParamsNew *pSrc, bool copyImages);

	int m_maxGenerations;
	bool m_deleteImages;
	double m_lensDistance, m_lensRedshift;
	double m_initialMassScale;
	std::list<ImagesDataExtended *> m_images;
	std::list<GridSquare> m_gridSquares;
	bool m_allowNegative;
	BasisFunctionType m_basisFunctionType;
	GravitationalLens *m_pBaseLens;
	MassSheetSearchType m_massSheetSearchType;

	bool m_showEvolution, m_saveFirstGenome;
};
	
} // end namespace

#endif // GRALE_GRIDLENSINVERSIONGAFACTORYPARAMSNEW_H
