/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "graleconfig.h"
#include "plummerbackprojecttree.h"
#include "constants.h"

#include "debugnew.h"

namespace grale
{

PlummerBackProjectTree::PlummerBackProjectTree()
{
}

PlummerBackProjectTree::~PlummerBackProjectTree()
{
}
	
void PlummerBackProjectTree::init(double D_d, double massScale, double widthScaleFactor,
		  const std::list<ImagesDataExtended *> &images,
		  Vector2D<double> center, double areaSize, 
		  int firstLevelSubdiv, int maxDepth, bool useWeights)
{
	m_scale = SQRT((4.0*CONST_G*massScale)/(SPEED_C*SPEED_C*D_d));
	m_widthScaleFactor = widthScaleFactor;

	BackProjectTreeBase::init(images, center, areaSize, firstLevelSubdiv, maxDepth, useWeights);
}

void PlummerBackProjectTree::getDeflectionInformation(double squareSize, Vector2D<double> theta, Vector2D<double> &alpha,
	                                           double &alphaxx, double &alphayy, double &alphaxy)
{
	double w = squareSize*m_widthScaleFactor/m_scale;
	double w2 = w*w;
	Vector2D<double> scaledTheta = theta/m_scale;
	double denom = scaledTheta.getLengthSquared() + w2;

	alpha = theta/denom;
	alphaxx = (scaledTheta.getY()*scaledTheta.getY() - scaledTheta.getX()*scaledTheta.getX() + w2)/(denom*denom);
	alphayy = (scaledTheta.getX()*scaledTheta.getX() - scaledTheta.getY()*scaledTheta.getY() + w2)/(denom*denom);
	alphaxy = (-2.0*scaledTheta.getX()*scaledTheta.getY())/(denom*denom);
}

} // end namespace

