/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_TREELENSINVERSIONGAFACTORYPARAMS_H

#define GRALE_TREELENSINVERSIONGAFACTORYPARAMS_H

#include "graleconfig.h"
#include "lensinversiongafactoryparams.h"
#include "grid.h"
#include <list>

namespace grale
{

class ImagesDataExtended;

class GRALE_IMPORTEXPORT TreeLensInversionGAFactoryParams : public LensInversionGAFactoryParams
{
public:
	enum BasisFunctionType { PlummerBasis };
	
	TreeLensInversionGAFactoryParams();
	TreeLensInversionGAFactoryParams(int maxGenerations,
					 const std::list<ImagesDataExtended *> &images, 
					 Vector2D<double> center,
					 double areaSize,
					 int maxDepth,
					 int maxBasisFunctions,
					 double D_d, 
					 double massScale,
					 bool copyImages,
					 double widthScaleFactor,
					 bool useWeights,
					 int firstSubdiv,
					 BasisFunctionType b = PlummerBasis);
	~TreeLensInversionGAFactoryParams();

	int getMaximumNumberOfGenerations() const								{ return m_maxGenerations; }
	double getD_d() const											{ return m_Dd; }
	double getMassScale() const										{ return m_massScale; }
	const std::list<ImagesDataExtended *> &getImages() const						{ return m_images; }
	BasisFunctionType getBasisFunctionType() const								{ return m_basisFunctionType; }
	Vector2D<double> getCenter() const									{ return m_center; }
	double getAreaSize() const										{ return m_areaSize; }
	int getMaxTreeDepth() const										{ return m_maxDepth; }
	int getMaxBasisFunctions() const									{ return m_maxBasisFunctions; }
	double getWidthScaleFactor() const									{ return m_widthScaleFactor; }
	bool useWeights() const											{ return m_useWeights; }
	int getInitialSubdivision() const									{ return m_firstSubdiv; }
	
	bool write(serut::SerializationInterface &si) const;
	bool read(serut::SerializationInterface &si);
private:
	int m_maxGenerations;
	bool m_deleteImages;
	double m_Dd, m_massScale;
	std::list<ImagesDataExtended *> m_images;
	BasisFunctionType m_basisFunctionType;
	Vector2D<double> m_center;
	double m_areaSize;
	double m_widthScaleFactor;
	int m_maxDepth, m_maxBasisFunctions;
	bool m_useWeights;
	int m_firstSubdiv;
};
	
} // end namespace

#endif // GRALE_TREELENSINVERSIONGAFACTORYPARAMS_H
