/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "graleconfig.h"
#include "real2dlensplanefunction.h"
#include "gravitationallens.h"
#include "constants.h"

#include "debugnew.h"

namespace grale
{

Real2DLensPlaneFunction::Real2DLensPlaneFunction(LensPlane &lensPlane) : m_lensPlane(lensPlane)
{
	m_densFactor = (SPEED_C*SPEED_C)/(lensPlane.getLens()->getLensDistance()*8.0*CONST_PI*CONST_G);
}

Real2DLensPlaneFunction::~Real2DLensPlaneFunction()
{
}

// TODO: improve this -> use interpolation
double Real2DLensPlaneFunction::operator()(Vector2D<double> theta) const
{
	double density = 0;
	double axx, ayy, axy;
	int x, y;
	
	if (!m_lensPlane.getIndices(theta, &x, &y))
		return 0;

	m_lensPlane.getAlphaDerivatives(x, y, axx, ayy, axy);
	density = m_densFactor*(axx+ayy);
	return density;
}

} // end namespace

