/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file rectangularintegrator.h
 */

#ifndef GRALE_RECTANGULARINTEGRATOR_H

#define GRALE_RECTANGULARINTEGRATOR_H

#include "graleconfig.h"
#include "real2dfunctionintegrator.h"
#include "uniformdistribution.h"

namespace grale
{

/** Integrate a 2D function over a rectangular area. */
class GRALE_IMPORTEXPORT RectangularIntegrator : public Real2DFunctionIntegrator
{
public:
	RectangularIntegrator(Vector2D<double> bottomLeft, Vector2D<double> topRight, int numX = 100, int numY = 100);
	~RectangularIntegrator();

	Vector2D<double> getTopRight() const 					{ return m_topRight; }
	Vector2D<double> getBottomLeft() const					{ return m_bottomLeft; }
	int getXSteps() const							{ return m_numX; }
	int getYSteps() const							{ return m_numY; }
	double getXStepSize() const						{ return m_xStep; }
	double getYStepSize() const						{ return m_yStep; }

	double integrate(Real2DFunction &f, bool abs = false) const;
private:
	Vector2D<double> m_topRight, m_bottomLeft;
	int m_numX, m_numY;
	double m_xStep, m_yStep;
};
	
} // end namespace

#endif // GRALE_RECTANGULARINTEGRATOR_H

