/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_POINTSOURCE_H

#define GRALE_POINTSOURCE_H

#include "graleconfig.h"
#include "sourceimage.h"

namespace grale
{

class GRALE_IMPORTEXPORT PointSource : public SourceImage
{
public:
	PointSource(Vector2D<double> angularpos, double brightnessScale);
	~PointSource();
	SourceImage *createCopy() const;
	bool isSerializable() const						{ return true; }
protected:
	PointSource();
	
	bool writeInternal(serut::SerializationInterface &si) const;
	bool readInternal(serut::SerializationInterface &si);
	double getIntensityInternal(Vector2D<double> diff) const		{ return 1.0; }
	double getMaxRadius() const						{ return 0; }
private:
	friend class SourceImage;
};

} // end namespace

#endif // GRALE_POINTSOURCE_H

